use glutin::{
    dpi::{PhysicalPosition, PhysicalSize},
    event_loop::EventLoop,
    window::Window as GlutinWindow,
    window::WindowBuilder,
    Api, ContextBuilder, ContextWrapper, GlProfile, GlRequest, PossiblyCurrent,
};

use crate::Error;

pub struct Window {
    context: ContextWrapper<PossiblyCurrent, GlutinWindow>,
}

impl Window {
    pub fn new<T>(event_loop: &EventLoop<T>) -> Result<Self, Error> {
        let window_builder = WindowBuilder::new()
            .with_title("space-crush")
            .with_visible(true)
            .with_inner_size(PhysicalSize::new(WINDOW_WIDTH, WINDOW_HEIGHT));
        let context = ContextBuilder::new()
            .with_double_buffer(Some(true))
            .with_hardware_acceleration(Some(true))
            .with_pixel_format(24, 8)
            .with_vsync(true)
            .with_gl(GlRequest::Specific(Api::OpenGl, (4, 5)))
            .with_gl_profile(GlProfile::Core)
            .build_windowed(window_builder, event_loop)?;

        let window = context.window();
        let window_size = window.outer_size();
        let monitor_size = window.current_monitor().unwrap().size();
        window.set_outer_position(PhysicalPosition::new(
            (monitor_size.width - window_size.width) / 2,
            (monitor_size.height - window_size.height) / 2,
        ));

        let context = unsafe { context.make_current().unwrap() };
        gl::load_with(|s| context.get_proc_address(s));

        Ok(Self { context })
    }

    pub fn swap_buffers(&self) -> Result<(), Error> {
        self.context.swap_buffers()?;

        Ok(())
    }
}

const WINDOW_WIDTH: u32 = 1280;
const WINDOW_HEIGHT: u32 = 720;
