mod misc;
mod render;
mod systems;

use specs::{Dispatcher, DispatcherBuilder, World};

use crate::Error;

use misc::{events::Events, geometry::Geometry, window::Window};
use render::Background;
use systems::{State, StateUpdate};

pub struct App<'a, 'b> {
    is_running: bool,
    events: Events,
    window: Window,
    dispatcher: Dispatcher<'a, 'b>,
}

impl<'a, 'b> App<'a, 'b> {
    pub fn new(world: &mut World) -> Result<Self, Error> {
        let events = Events::new(world);
        let window = Window::new(events.handle())?;

        let mut dispatcher = DispatcherBuilder::new()
            .with(StateUpdate::new(world), "state_update", &[])
            .with_thread_local(Background::new(world)?)
            .build();
        dispatcher.setup(world);

        world.insert(Geometry::new()?);

        Ok(Self {
            is_running: true,
            events,
            window,
            dispatcher,
        })
    }

    pub fn is_running(&self) -> bool {
        self.is_running
    }

    pub fn process(&mut self, world: &World) -> Result<(), Error> {
        self.events.process(world);
        self.dispatcher.dispatch(world);

        self.window.swap_buffers()?;

        self.is_running = !world.fetch::<State>().close_requested;

        Ok(())
    }
}
