use glc::{
    misc::Bindable,
    shader::{Program, Shader, Type},
};
use shrev::{EventChannel, ReaderId};
use specs::{ReadExpect, System, World};

use crate::Error;

use super::super::misc::{events::WindowEvent, geometry::Geometry};

pub struct Background {
    program: Program,
    window_events_id: ReaderId<WindowEvent>,
}

impl Background {
    pub fn new(world: &mut World) -> Result<Self, Error> {
        let shaders = vec![
            (Type::Vertex, include_str!("shader/noise.vert")),
            (Type::Fragment, include_str!("shader/noise.frag")),
        ];
        let program = Program::from_shaders_result(
            shaders
                .into_iter()
                .map(|(t, s)| Shader::from_string(t, s.into())),
        )?;
        let window_events_id = world
            .fetch_mut::<EventChannel<WindowEvent>>()
            .register_reader();

        Ok(Self {
            program,
            window_events_id,
        })
    }
}

impl<'a> System<'a> for Background {
    type SystemData = (
        ReadExpect<'a, Geometry>,
        ReadExpect<'a, EventChannel<WindowEvent>>,
    );

    fn run(&mut self, (geometry, window_events): Self::SystemData) {
        let events = window_events.read(&mut self.window_events_id);
        for event in events {
            if let WindowEvent::Resize(w, h) = event {
                gl::viewport(0, 0, *w as gl::GLsizei, *h as gl::GLsizei);
            }
        }

        self.program.bind();
        geometry.render_quad();
        self.program.unbind();
    }
}
