use shrev::{EventChannel, ReaderId};
use specs::{ReadExpect, System, World, Write};

use super::super::misc::events::WindowEvent;

/* State */

#[derive(Default)]
pub struct State {
    pub close_requested: bool,

    pub window_width: u32,
    pub window_height: u32,
}

/* StateUpdate*/

pub struct StateUpdate {
    window_events_id: ReaderId<WindowEvent>,
}

impl StateUpdate {
    pub fn new(world: &mut World) -> Self {
        let window_events_id = world
            .fetch_mut::<EventChannel<WindowEvent>>()
            .register_reader();

        Self { window_events_id }
    }
}

impl<'a> System<'a> for StateUpdate {
    type SystemData = (Write<'a, State>, ReadExpect<'a, EventChannel<WindowEvent>>);

    fn run(&mut self, (mut state, window_events): Self::SystemData) {
        let events = window_events.read(&mut self.window_events_id);
        for event in events {
            match event {
                WindowEvent::Resize(w, h) => {
                    state.window_width = *w;
                    state.window_height = *h;
                }
                WindowEvent::Close => {
                    state.close_requested = true;
                }
            }
        }
    }
}
