use glc::error::Error as GlcError;
use glutin::{ContextError as GlutinContextError, CreationError as GlutinCreationError};
use thiserror::Error;

#[derive(Debug, Error)]
pub enum Error {
    #[error("GLC Error: {0}")]
    GlcError(GlcError),

    #[error("glutin Context Error: {0}")]
    GlutinContextError(GlutinContextError),

    #[error("glutin Creation Error: {0}")]
    GlutinCreationError(GlutinCreationError),
}

impl From<GlcError> for Error {
    fn from(err: GlcError) -> Self {
        Self::GlcError(err)
    }
}

impl From<GlutinContextError> for Error {
    fn from(err: GlutinContextError) -> Self {
        Self::GlutinContextError(err)
    }
}

impl From<GlutinCreationError> for Error {
    fn from(err: GlutinCreationError) -> Self {
        Self::GlutinCreationError(err)
    }
}
