use log::{error, info};
use specs::{World, WorldExt};

use space_crush::{App, Error, Server};

fn main() -> Result<(), Error> {
    env_logger::builder()
        .filter_level(log::LevelFilter::Trace)
        .format_timestamp_nanos()
        .init();

    info!("Application started");

    if let Err(err) = run() {
        error!("Error while executing application: {}", err);

        return Err(err);
    }

    info!("Application exited");

    Ok(())
}

fn run() -> Result<(), Error> {
    let mut world = World::new();
    let mut server = Server::new(&mut world);
    let mut app = App::new(&mut world)?;

    while app.is_running() {
        server.process(&world);
        app.process(&world)?;
    }

    Ok(())
}
